from eve_jwt import requires_token
from eve_jwt.auth import JWTAuth


def register(app):
    @app.route('/token/success')
    @requires_token(audiences=['aud1'], allowed_roles=['super'])
    def requires_token_success():
        return 'true'

    @app.route('/token/failure')
    @requires_token(audiences=['aud1'], allowed_roles=['super'])
    def requires_token_failure():
        return 'should not authenticate'

    custom_auth = JWTAuth()
    custom_auth.issuer = 'custom_issuer'

    @app.route('/custom/success')
    @custom_auth.requires_token(audiences=['aud1'], allowed_roles=['super'])
    def requires_token_success2():
        return 'true'



"""Top-level package for eve-jwt."""

__author__ = """Yossi Mosbacher"""
__email__ = 'joe.mosbacher@gmail.com'
__version__ = '0.1.9'

from .validation import AsymmetricKeyValidator
from .auth import JWTAuth



# -*- coding: utf-8 -*-

from eve.auth import BasicAuth, TokenAuth
from eve.utils import config
from flask import request, Response, g
from flask import abort
from functools import wraps
from .validation import AsymmetricKeyValidator


AUTH_CLAIMS = 'authen_claims'
AUTH_ROLES = 'authen_roles'
AUTH_VALUE = 'auth_value'


class JWTAuth(TokenAuth):
    """
    Implements JWT token validation support.
    """

    def __init__(self, validator=None ,issuer=None):
        self.validator = validator #AsymmetricKeyValidator(key_url=key_url, default_key=default_key)
        self.issuer = issuer

    @property
    def validator(self):
        if self._validator is None:
            scope_claim = config.JWT_SCOPE_CLAIM
            roles_claim = config.JWT_ROLES_CLAIM
            self._validator = AsymmetricKeyValidator(key_url=config.JWT_KEY_URL,
                                        scope_claim=scope_claim, roles_claim=roles_claim)
        return self._validator

    @validator.setter
    def validator(self, value):
        self._validator = value

    @property
    def issuer(self):
        if self._issuer is None:
            return config.JWT_ISSUER
        return self._issuer

    @issuer.setter
    def issuer(self, value):
        self._issuer = value

    def set_authen_claims(self, claims):
        setattr(g, AUTH_CLAIMS, claims)

    def get_authen_claims(self):
        return g.get(AUTH_CLAIMS, {})

    def set_authen_roles(self, roles):
        setattr(g, AUTH_ROLES, roles)

    def get_authen_roles(self):
        return g.get(AUTH_ROLES, [])

    def authorized(self, allowed_roles, resource, method):
        authorized = False

        if request.authorization:
            auth = request.authorization
            authorized = self.check_auth(auth.username, auth.password,
                                         allowed_roles, resource, method)
        else:
            try:
                access_token = request.args['access_token']
            except KeyError:
                access_token = request.headers.get('Authorization', '').partition(' ')[2]
            authorized = self.check_token(access_token, allowed_roles, resource, method)

        return authorized

    def authenticate(self):
        """
        Indicate to the client that it needs to authenticate via a 401.
        """
        if request.headers.get('Authorization') or request.args.get('access_token'):
            realm = 'Bearer realm="%s", error="invalid_token"' % __package__
        else:
            realm = 'Bearer realm="%s"' % __package__
        # print(realm)
        resp = Response(None, 401, {'WWW-Authenticate': realm})
        abort(401, description='Please provide proper credentials', response=resp)

    def check_token(self, token, allowed_roles, resource, method):
        """
        This function is called when a token is sent throught the access_token
        parameter or the Authorization header as specified in the oAuth 2 specification.

        The provided token is validated with the JWT_SECRET defined in the Eve configuration.
        The token issuer (iss claim) must be the one specified by JWT_ISSUER and the audience
        (aud claim) must be one of the value(s) defined by the either the "audiences" resource
        parameter or the global JWT_AUDIENCES configuration.

        If JWT_ROLES_CLAIM is defined and a claim by that name is present in the token, roles
        are checked using this claim.

        If a JWT_SCOPE_CLAIM is defined and a claim by that name is present in the token, the
        claim value is check, and if "viewer" is present, only GET and HEAD methods will be
        allowed. The scope name is then added to the list of roles with the scope: prefix.

        If the validation succeed, the claims are stored and accessible thru the
        get_authen_claims() method.
        """
        domain = config.DOMAIN or {}
        resource_conf = domain.get(resource, {})
        audiences = resource_conf.get('audiences', config.JWT_AUDIENCES)
        return self._perform_validation(token, audiences, allowed_roles)

    def requires_token(self, audiences=None, allowed_roles=None):
        """
        Decorator for functions that will be protected with token authentication.

        Token must be provvided either through access_token parameter or Authorization
        header.

        See check_token() method for further details.
        """
        def requires_token_wrapper(f):
            @wraps(f)
            def decorated(*args, **kwargs):
                try:
                    token = request.args['access_token']
                except KeyError:
                    token = request.headers.get('Authorization', '').partition(' ')[2]

                if not self._perform_validation(token, audiences, allowed_roles):
                    self.authenticate()
                    # abort(401)

                return f(*args, **kwargs)
            return decorated
        return requires_token_wrapper

    def _perform_validation(self, token, audiences, allowed_roles):
        validated, payload, account_id, roles = self.validator.validate_token(
                token, self.issuer, request.method, audiences, allowed_roles)
        if not validated:
            return False

        # Save roles for later access
        self.set_authen_roles(roles)

        # Save claims for later access
        self.set_authen_claims(payload)

        # Limit access to the authen account
        self.set_request_auth_value(account_id)

        return True

    def check_auth(self, username, password, allowed_roles, resource, method):
        return False


requires_token = JWTAuth().requires_token


def set_authen_claims(claims):
    """
    Set the authentication claims

    Parameters:
        claims (dict[str]): JWT claims
    """
    setattr(g, AUTH_CLAIMS, claims)


def get_authen_claims():
    """
    Get the authentication claims

    Returns:
        dict[str]: JWT claims
    """
    return g.get(AUTH_CLAIMS, {})


def set_authen_roles(roles=[]):
    """
    Get the authentication roles

    Parameters:
        roles (arr[str])
    """
    setattr(g, AUTH_ROLES, roles)


def get_authen_roles():
    """
    Get the authentication roles

    Returns:
        arr[str]: Array of associated roles
    """
    return g.get(AUTH_ROLES, [])


def set_request_auth_value(value=None):
    """
    Sets the current request's auth value

    Parameters:
        value (str|None): The request auth value
    """
    setattr(g, AUTH_VALUE, value)


def get_request_auth_value():
    """
    Get the authentication value

    Returns:
        str: auth value string
    """
    return g.get(AUTH_VALUE)



# -*- coding: utf-8 -*-

import unittest
from authlib.jose import jwt, jwk
from eve import Eve
from eve_jwt import JWTAuth
from flask import g
from . import test_routes
from cryptography.hazmat.primitives.asymmetric import rsa
from cryptography.hazmat.primitives import serialization

private_key = rsa.generate_private_key(
    public_exponent=65537,
   key_size=2048,
)
public_key = private_key.public_key()

alt_private_key = rsa.generate_private_key(
    public_exponent=65537,
   key_size=2048,
)
alt_public_key = private_key.public_key()
# pem = public_key.public_bytes(
#     encoding=serialization.Encoding.PEM,
#     format=serialization.PublicFormat.SubjectPublicKeyInfo
#  )
header = {'alg': 'RS256', 'kid':'default'}

settings = {
    'JWT_DEFAULT_KEY': jwk.dumps(public_key),
    'JWT_KEY_URL': '',
    'JWT_TTL': 600,
    'JWT_ISSUER': 'https://domain.com/token',
    'JWT_ROLES_CLAIM': 'roles',
    'JWT_SCOPE_CLAIM': 'scope',
    'JWT_AUDIENCES': [],
    'DOMAIN': {
        'foo': {
            'schema': {
                'name': {},
            },
            'audiences': ['aud1'],
            'resource_methods': ['POST', 'GET'],
        },
        'bar': {
            'audiences': ['aud2'],
        },
        'baz': {
            'audiences': ['aud1'],
            'allowed_roles': ['role1', 'role2'],
        },
        'bad': {
        },
        'bag': {
            'audiences': ['aud1'],
            # 'authentication': JWTAuth(jwk.dumps(alt_public_key)),
        },
    },
}


class TestBase(unittest.TestCase):
    def setUp(self):
        self.app = Eve(settings=settings, auth=JWTAuth(default_key=public_key))
        test_routes.register(self.app)
        self.test_client = self.app.test_client()

    def test_restricted_access(self):
        r = self.test_client.get('/foo')
        self.assertEqual(r.status_code, 401)
        self.assertEqual(r.headers['WWW-Authenticate'], 'Bearer realm="eve_jwt"')

    def test_token_error(self):
        r = self.test_client.get('/foo?access_token=invalid')
        self.assertEqual(r.status_code, 401)
        self.assertEqual(r.headers['WWW-Authenticate'], 'Bearer realm="eve_jwt", error="invalid_token"')

    def test_valid_token_header(self):
        claims = {'iss': 'https://domain.com/token',
                  'aud': 'aud1',
                  'sub': '0123456789abcdef01234567'}
        token = jwt.encode(header, claims, private_key)
        auth = [('Authorization', 'Bearer {}'.format(token.decode('utf-8')))]
        r = self.test_client.get('/foo', headers=auth)
        self.assertEqual(r.status_code, 200)

    def test_valid_token_query(self):
        claims = {'iss': 'https://domain.com/token',
                  'aud': 'aud1',
                  'sub': '0123456789abcdef01234567'}
        token = jwt.encode(header, claims, private_key)
        r = self.test_client.get('/foo?access_token={}'.format(token.decode('utf-8')))
        self.assertEqual(r.status_code, 200)

    def test_token_claims_context(self):
        claims = {'iss': 'https://domain.com/token',
                  'aud': 'aud1',
                  'sub': '0123456789abcdef01234567'}
        token = jwt.encode(header, claims, private_key)
        with self.app.test_client() as client:
            client.get('/foo?access_token={}'.format(token.decode('utf-8')))
            self.assertEqual(g.get('authen_claims'), claims)

    def test_invalid_token_secret(self):
        claims = {'iss': 'https://domain.com/token',
                  'aud': 'aud1',
                  'sub': '0123456789abcdef01234567'}
        token = jwt.encode(header, claims, alt_private_key)
        r = self.test_client.get('/foo?access_token={}'.format(token.decode('utf-8')))
        self.assertEqual(r.status_code, 401)

    def test_missing_token_subject(self):
        claims = {'iss': 'https://domain.com/token',
                  'aud': 'aud1'}
        token = jwt.encode(header, claims, private_key)
        r = self.test_client.get('/foo?access_token={}'.format(token.decode('utf-8')))
        self.assertEqual(r.status_code, 200)

    def test_invalid_token_issuer(self):
        claims = {'iss': 'https://invalid-domain.com/token',
                  'aud': 'aud1',
                  'sub': '0123456789abcdef01234567'}
        token = jwt.encode(header, claims, private_key)
        r = self.test_client.get('/foo?access_token={}'.format(token.decode('utf-8')))
        self.assertEqual(r.status_code, 401)

    def test_invalid_token_audience(self):
        claims = {'iss': 'https://domain.com/token',
                  'aud': 'aud2',
                  'sub': '0123456789abcdef01234567'}
        token = jwt.encode(header, claims, private_key)
        r = self.test_client.get('/foo?access_token={}'.format(token.decode('utf-8')))
        self.assertEqual(r.status_code, 401)

    def test_valid_token_resource_audience(self):
        claims = {'iss': 'https://domain.com/token',
                  'aud': 'aud2',
                  'sub': '0123456789abcdef01234567'}
        token = jwt.encode(header, claims, private_key)
        r = self.test_client.get('/bar?access_token={}'.format(token.decode('utf-8')))
        self.assertEqual(r.status_code, 200)

    def test_invalid_token_resource_audience(self):
        claims = {'iss': 'https://domain.com/token',
                  'aud': 'aud1',
                  'sub': '0123456789abcdef01234567'}
        token = jwt.encode(header, claims, private_key)
        r = self.test_client.get('/bar?access_token={}'.format(token.decode('utf-8')))
        self.assertEqual(r.status_code, 401)

    def test_valid_token_role(self):
        claims = {'iss': 'https://domain.com/token',
                  'aud': 'aud1',
                  'sub': '0123456789abcdef01234567',
                  'roles': ['role1']}
        token = jwt.encode(header, claims, private_key)
        r = self.test_client.get('/baz?access_token={}'.format(token.decode('utf-8')))
        self.assertEqual(r.status_code, 200)

    def test_invalid_token_role(self):
        claims = {'iss': 'https://domain.com/token',
                  'aud': 'aud1',
                  'sub': '0123456789abcdef01234567'}
        token = jwt.encode(header, claims, private_key)
        r = self.test_client.get('/baz?access_token={}'.format(token.decode('utf-8')))
        self.assertEqual(r.status_code, 401)

    def test_token_role_context(self):
        claims = {'iss': 'https://domain.com/token',
                  'aud': 'aud1',
                  'sub': '0123456789abcdef01234567',
                  'roles': ['role1']}
        token = jwt.encode(header, claims, private_key)
        with self.app.test_client() as client:
            client.get('/baz?access_token={}'.format(token.decode('utf-8')))
            self.assertEqual(g.get('authen_roles'), ['role1'])

    def test_token_role_context_always(self):
        claims = {'iss': 'https://domain.com/token',
                  'aud': 'aud1',
                  'sub': '0123456789abcdef01234567',
                  'roles': ['role1']}
        token = jwt.encode(header, claims, private_key)
        with self.app.test_client() as client:
            client.get('/foo?access_token={}'.format(token.decode('utf-8')))
            self.assertEqual(g.get('authen_roles'), ['role1'])

    def test_token_scope(self):
        claims = {'iss': 'https://domain.com/token',
                  'aud': 'aud1',
                  'sub': '0123456789abcdef01234567',
                  'scope': 'user'}
        token = jwt.encode(header, claims, private_key)
        r = self.test_client.get('/foo?access_token={}'.format(token.decode('utf-8')))
        self.assertEqual(r.status_code, 200)

    def test_token_scope_viewer_read(self):
        claims = {'iss': 'https://domain.com/token',
                  'aud': 'aud1',
                  'sub': '0123456789abcdef01234567',
                  'scope': 'viewer'}
        token = jwt.encode(header, claims, private_key)
        r = self.test_client.get('/foo?access_token={}'.format(token.decode('utf-8')))
        self.assertEqual(r.status_code, 200)

    def test_token_scope_viewer_write(self):
        claims = {'iss': 'https://domain.com/token',
                  'aud': 'aud1',
                  'sub': '0123456789abcdef01234567',
                  'scope': 'viewer'}
        token = jwt.encode(header, claims, private_key)
        r = self.test_client.post('/foo?access_token={}'.format(token.decode('utf-8')))
        self.assertEqual(r.status_code, 401)

    def test_requires_token_success(self):
        claims = {'iss': 'https://domain.com/token',
                  'aud': 'aud1',
                  'sub': '0123456789abcdef01234567',
                  'roles': ['super'],
                  'scope': 'user'}
        token = jwt.encode(header, claims, private_key)
        r = self.test_client.get('/token/success?access_token={}'.format(token.decode('utf-8')))
        self.assertEqual(r.status_code, 200, r.data)

    def test_requires_token_failure_audience(self):
        claims = {'iss': 'https://domain.com/token',
                  'aud': 'aud2',
                  'sub': '0123456789abcdef01234567',
                  'roles': ['super'],
                  'scope': 'user'}
        token = jwt.encode(header, claims, private_key)
        r = self.test_client.get('/token/failure?access_token={}'.format(token.decode('utf-8')))
        self.assertEqual(r.status_code, 401, r.data)

    def test_requires_token_failure_roles(self):
        claims = {'iss': 'https://domain.com/token',
                  'aud': 'aud1',
                  'sub': '0123456789abcdef01234567',
                  'roles': [],
                  'scope': 'user'}
        token = jwt.encode(header, claims, private_key)
        r = self.test_client.get('/token/failure?access_token={}'.format(token.decode('utf-8')))
        self.assertEqual(r.status_code, 401, r.data)

    def test_auth_header_token_success(self):
        claims = {'iss': 'https://domain.com/token',
                  'aud': 'aud1',
                  'sub': '0123456789abcdef01234567',
                  'roles': ['super'],
                  'scope': 'user'}
        token = jwt.encode(header, claims, private_key)
        headers = {'Authorization': 'Bearer {}'.format(token.decode('utf-8'))}
        r = self.test_client.get('/token/success', headers=headers)
        self.assertEqual(r.status_code, 200)

    def test_auth_header_token_failure(self):
        r = self.test_client.get('/token/failure')
        self.assertEqual(r.status_code, 401, r.data)

    def test_no_audience_token_success(self):
        claims = {'iss': 'https://domain.com/token'}
        token = jwt.encode(header, claims, private_key)
        r = self.test_client.get('/bad?access_token={}'.format(token.decode('utf-8')))
        self.assertEqual(r.status_code, 200)

    def test_no_audience_token_failure(self):
        claims = {'iss': 'https://domain.com/token',
                  'aud': 'aud1',
                  'sub': '0123456789abcdef01234567'}
        token = jwt.encode(header, claims, private_key)
        r = self.test_client.get('/bad?access_token={}'.format(token.decode('utf-8')))
        self.assertEqual(r.status_code, 401)

    def test_endpoint_level_auth_with_different_secret(self):
        claims = {'iss': 'https://domain.com/token',
                  'aud': 'aud1'}
        token = jwt.encode(header, claims, private_key)
        r = self.test_client.get('/bag?access_token={}'.format(token.decode('utf-8')))
        self.assertEqual(r.status_code, 200)

    def test_auth_header_token_success_with_different_secret(self):
        claims = {'iss': 'custom_issuer',
                  'aud': 'aud1',
                  'roles': ['super']}
        token = jwt.encode(header, claims, private_key)
        headers = {'Authorization': 'Bearer {}'.format(token.decode('utf-8'))}
        r = self.test_client.get('/custom/success', headers=headers)
        self.assertEqual(r.status_code, 200)

    def test_auth_header_token_failure_with_wrong_issuer(self):
        claims = {'iss': 'https://domain.com/token',
                  'aud': 'aud1',
                  'roles': ['super']}
        token = jwt.encode(header, claims, private_key)
        headers = {'Authorization': 'Bearer {}'.format(token.decode('utf-8'))}
        r = self.test_client.get('/custom/success', headers=headers)
        self.assertEqual(r.status_code, 401)


if __name__ == '__main__':
    unittest.main()



import requests
from authlib.jose import jwt, jwk, util, errors, JsonWebKey, KeySet
import authlib
import logging

logger = logging.getLogger(__name__)

class ValidatorBase:
    def validate_token(self, token, issuer, method=None, 
                        audiences=None, allowed_roles=None):
        raise NotImplementedError


class AsymmetricKeyValidator(ValidatorBase):
    def __init__(self, default_key=None, key_url=None,
                 scope_claim=None, roles_claim=None):
        self.key_url = key_url
        self.roles_claim = roles_claim
        self.scope_claim = scope_claim
        self._keyset = KeySet([])


    def validate_token(self, token, issuer, method=None, audiences=None, allowed_roles=None):
        key = self.get_key(token)
        if not key:
            return (False, None, None, None)

        options = {}

        if audiences:
            if isinstance(audiences, str):
                options["aud"] = {"essential": True, "value": audiences}
            else:
                options["aud"] = {"essential": True, "values": audiences}
        else:
            options["aud"] = {"essential": False, "values": []}
            
        if issuer:
            options["iss"] = {"essential": True, "value": issuer}

        try:
            claims = jwt.decode(token, key, claims_options=options)
            claims.validate()
        except:
            return (False, None, None, None)
        
        payload = dict(claims)
        account_id = payload.get('sub')  # Get account id

        roles = None

        # Check scope is configured and add append it to the roles
        if self.scope_claim and payload.get(self.scope_claim):
            scope = payload.get(self.scope_claim)
            roles = scope.split(" ")

        # If roles claim is defined, gather roles from the token
        if self.roles_claim:
            roles = payload.get(self.roles_claim, []) + (roles or [])

        # Check roles if scope or role claim is set
        if allowed_roles and roles is not None:
            if not any(role in roles for role in allowed_roles):
                return (False, payload, account_id, roles)

        return (True, payload, account_id, roles)

    def get_key(self, token):
        kid = ""
        try:
            header_str = authlib.common.encoding.urlsafe_b64decode(token.split(".")[0].encode()).decode('utf-8')
            header = authlib.common.encoding.json_loads(header_str)
            kid = header["kid"]
            return self._keyset.find_by_kid(kid)
        except Exception as e:
            logger.debug(str(e))
            kid = ""
        try:
            self.fetch_keys()
            return self._keyset.find_by_kid(kid)
        except Exception as e:
            logger.debug(str(e))
            return False


    def fetch_keys(self):
        if not self.key_url:
            return
        response = requests.get(self.key_url)
        if response.ok:
            data = response.json()
            self._keyset = JsonWebKey.import_key_set(data)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['eve_jwt', 'tests']

package_data = \
{'': ['*']}

install_requires = \
['Authlib>=0.15.2,<0.16.0',
 'eve>=1.1.4,<2.0.0',
 'flask>=1.1.2,<2.0.0',
 'requests>=2.25.0,<3.0.0']

setup_kwargs = {
    'name': 'eve-jwt',
    'version': '0.1.9',
    'description': 'Top-level package for eve-jwt.',
    'long_description': '=======\neve-jwt\n=======\n\n\n.. image:: https://img.shields.io/pypi/v/eve_jwt.svg\n        :target: https://pypi.python.org/pypi/eve_jwt\n\n.. image:: https://img.shields.io/travis/jmosbacher/eve_jwt.svg\n        :target: https://travis-ci.com/jmosbacher/eve_jwt\n\n.. image:: https://readthedocs.org/projects/eve-jwt/badge/?version=latest\n        :target: https://eve-jwt.readthedocs.io/en/latest/?badge=latest\n        :alt: Documentation Status\n\n\n\nEve JWT auth for asymmetric encryption. Adapted from eve-auth-jwt which handles symmetric encryption shcemes.\n\n\nUsage\n-----\n\n.. code-block:: python\n\n        from eve_jwt import JWTAuth\n        from eve import Eve\n\n        auth = JWTAuth(key_url="https://YOUR-OAUTH-SERVER/certs/")\n\n        app = Eve(auth=auth)\n\n\n* Free software: MIT\n* Documentation: https://eve-jwt.readthedocs.io.\n\n\nFeatures\n--------\n\n* TODO\n\nCredits\n-------\n\nThis package was created with Cookiecutter_ and the `briggySmalls/cookiecutter-pypackage`_ project template.\n\n.. _Cookiecutter: https://github.com/audreyr/cookiecutter\n.. _`briggySmalls/cookiecutter-pypackage`: https://github.com/briggySmalls/cookiecutter-pypackage\n',
    'author': 'Yossi Mosbacher',
    'author_email': 'joe.mosbacher@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/jmosbacher/eve_jwt',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6',
}


setup(**setup_kwargs)
